/*Includes ----------------------------------------------*/
#include "system.h"
#include "exception.h"
#include "app_charge.h"
#include "app_flash.h"
#include "app_self_checking.h"
#include "hal_led.h"
#include "hal_mt.h"
#include "hal_ble_uart0.h"
#include "hal_ble_client.h"
#include "hal_ser_imu_mode_manage.h"
#include "app_flash.h"
#include "app_pair.h"
#include "bsp_time.h"
#include "hal_scan_manage.h"
#include "ble_comm.h"

/*Private macro ------------------------------------------------*/
#define	SELF_CHECK_WHEEL_EMPTY_TIMES_MAX		MONITOR_SUSPEND_MODE_OVERFLOW_ERR_SUM_MAX			//ֿմ
#define SELF_CHECK_ITEMS										6																							//Լ
#define SELF_CHECK_ITEM_SUCCESS							0																							//ԼĿɹ
#define SELF_CHECK_ITEM_FAIL								1																							//ԼĿʧ

#define SELF_CHECK_ITEM_FRONT_SENSOR_INDEX	0																							//ԼĿ - ǰŴ
#define SELF_CHECK_ITEM_BACK_SENSOR_INDEX		1																							//ԼĿ - Ŵ
#define SELF_CHECK_ITEM_BATTERY_INDEX				2																							//ԼĿ - 
#define	SELF_CHECK_ITEM_CHARGE_INDEX				3																							//ԼĿ - ѹ
#define SELF_CHECK_ITEM_ANT_INDEX						4																							//ԼĿ - 
#define SELF_CHECK_ITEM_SUSPEND_MODE_INDEX	5																							//ԼĿ - 쳣			

#define SELF_CHECK_PROCESS_CYCLE						10																						//Լ߳ - msΪλ

#define SELF_CHECK_LED_DISPLAY_TIME					3000					  //Լʾʱ - msΪλ

#define SELF_CHECK_LED_TOGGLE_TIME					200						  //ԼƷתʱ - msΪλ

#define SELF_CHECK_SCAN_TRIGGER_TIMEOUT			10000						//ɨ败Լ쳬ʱʱ - msΪλ

#define SELF_CHECK_SET_IMU_TIMEOUT			    20000						//ȴIMUʼɹʱ

#define SELF_CHECK_SCAN_TARGET_ADV_NAME			"SH_SELF_CHECK"																//ɨ败Ŀ㲥

/*STRUCTION --------------------------------------------------
--*/
typedef enum {
	SELF_CHECK_STAGE_READY,																																	//׼׶
	SELF_CHECK_STAGE_CHECK,																																	//Լ׶
	
} SELF_CHECK_STAGE_e;
typedef enum{
	SELF_CHECK_LED_ON,																																			//Լ - 
	SELF_CHECK_LED_OFF,																																			//Լ - 
	
} SELF_CHECK_LED_SWITCH_e;
typedef struct self_check
{
	/*private member*/
	SELF_CHECK_STAGE_e				stage;																												//׶ֶ
	
	uint32_t								  ImuSetTime_times;																						  //ֿմ
	
	uint8_t										result[SELF_CHECK_ITEMS];																			//Լ
	
	uint8_t										item_fail_num;																								//ԼĿʧ
	
	SELF_CHECK_LED_SWITCH_e		led_switch;																										//Լƿ

	uint32_t									led_display_time;																							//Լʾʱ
	
	uint32_t									led_toggle_time;																							//ԼƷתʱ
	
	uint32_t									scan_trigger_timeout;																					//ɨ败Լ쳬ʱʱ
	
	bool											is_scan_success;																							//Ƿҵ㲥

} Self_Check_t;

/*Local Variable -----------------------------------------------*/
static Self_Check_t ob_self_check;

/*Local Functions ----------------------------------------------*/
static void clear_all_except(void)
{
	Except_ClearExceptype(EXCEPT_DATA_BACK_MAG);
	Except_ClearExceptype(EXCEPT_DATA_FRONT_ACC);
	Except_ClearExceptype(EXCEPT_DATA_FRONT_GRY);
	Except_ClearExceptype(EXCEPT_DATA_FRONT_MAG);
	Except_ClearExceptype(EXCEPT_DATA_CHARGE);
	Except_ClearExceptype(EXCEPT_DATA_BATTERY);
	Except_ClearExceptype(EXCEPT_IMU_SUSPEND_OVERFLOW);
	Except_ClearExceptype(EXCEPT_ANT_DAMAGE);
}

static void selft_scan_cb(void)
{
	ob_self_check.is_scan_success = true;
}

static scan_reslt_t scan_reslt_handle ={
	.handle = selft_scan_cb,
	.scanname =SELF_CHECK_SCAN_TARGET_ADV_NAME,
};

static uint8_t self_uart_check_flag =0;
static void cb_UART0_R_LINE_CHECK_ACK(void* handle)
{	
	//յӦ
	if(!mFlash.isHost) return; //ӻԷ
	self_uart_check_flag = 1;
}

static void cb_UART0_R_LINE_CHECK_ASK(void* handle)
{	//ӻյ
  uint32_t 		txd,rxd;
	if(mFlash.isHost){
		return; //ֹԷ
	}
	self_uart_check_flag =1;
	
	UART0_GetPinConfig(&txd, &rxd);
	UART0_Initialize(PIN_TXD_BLE,UART0_INVALID_PIN,UART_HZ);
	UART0_Config_Send(UART0_T_LINE_CHECK_ACK);//Ӧ
	if(rxd == 0 && txd == 0){txd = UART0_INVALID_PIN;rxd = PIN_RXD_BLE;}
	UART0_Initialize(txd,rxd,UART_HZ);
}


static void app_UartLine_Pair_Process(){
	  static uint8_t state =0;
	  uint32_t 		txd,rxd;
	  switch(state){
			case 0:
				    if(true == ob_self_check.is_scan_success){
							 state =1;
							 self_uart_check_flag =0;
						}
				    break;
			case 1:
				    if(mFlash.isHost){
							UART0_GetPinConfig(&txd, &rxd);
							UART0_Initialize(PIN_TXD_BLE,UART0_INVALID_PIN,UART_HZ);
							UART0_Config_Send(UART0_T_LINE_CHECK_ASK);
							if(rxd == 0 && txd == 0){txd = UART0_INVALID_PIN;rxd = PIN_RXD_BLE;}
							UART0_Initialize(txd,rxd,UART_HZ);
					  } 
			      if(false == ob_self_check.is_scan_success){
							  state = 0;						
						}
				    break;
			default:
				    break;
		}
}


static void app_self_check_getreslut(){
		/*ȡԼǰ---------------------------------------------------------------------*/
		ob_self_check.result[SELF_CHECK_ITEM_FRONT_SENSOR_INDEX] = SELF_CHECK_ITEM_SUCCESS;
		ob_self_check.result[SELF_CHECK_ITEM_BACK_SENSOR_INDEX] = SELF_CHECK_ITEM_SUCCESS;
		ob_self_check.result[SELF_CHECK_ITEM_BATTERY_INDEX] = SELF_CHECK_ITEM_SUCCESS;
		ob_self_check.result[SELF_CHECK_ITEM_CHARGE_INDEX] = SELF_CHECK_ITEM_SUCCESS;
		ob_self_check.result[SELF_CHECK_ITEM_ANT_INDEX] = SELF_CHECK_ITEM_SUCCESS;
		ob_self_check.result[SELF_CHECK_ITEM_SUSPEND_MODE_INDEX] = SELF_CHECK_ITEM_SUCCESS;						
		ob_self_check.item_fail_num = 0;

		/*ȡԼ---------------------------------------------------------------------*/
		
		//Լ - 쳣
		if(Except_IsError(EXCEPT_IMU_SUSPEND_OVERFLOW)){ob_self_check.result[SELF_CHECK_ITEM_SUSPEND_MODE_INDEX] = SELF_CHECK_ITEM_FAIL;ob_self_check.item_fail_num = SELF_CHECK_ITEM_SUSPEND_MODE_INDEX + 1;}	
		//Լ - ǰŴ
		if(Except_IsError(EXCEPT_DATA_FRONT_ACC)){ob_self_check.result[SELF_CHECK_ITEM_FRONT_SENSOR_INDEX] = SELF_CHECK_ITEM_FAIL;}
		if(Except_IsError(EXCEPT_DATA_FRONT_GRY)){ob_self_check.result[SELF_CHECK_ITEM_FRONT_SENSOR_INDEX] = SELF_CHECK_ITEM_FAIL;}
		if(Except_IsError(EXCEPT_DATA_FRONT_MAG)){ob_self_check.result[SELF_CHECK_ITEM_FRONT_SENSOR_INDEX] = SELF_CHECK_ITEM_FAIL;}
		if(Except_IsError(EXCEPT_DATA_FRONT_ACC) || Except_IsError(EXCEPT_DATA_FRONT_GRY) || Except_IsError(EXCEPT_DATA_FRONT_MAG)){ob_self_check.item_fail_num = SELF_CHECK_ITEM_FRONT_SENSOR_INDEX + 1;}
		//Լ - Ŵ
		if(Except_IsError(EXCEPT_DATA_BACK_MAG)){ob_self_check.result[SELF_CHECK_ITEM_BACK_SENSOR_INDEX] = SELF_CHECK_ITEM_FAIL;ob_self_check.item_fail_num = SELF_CHECK_ITEM_BACK_SENSOR_INDEX + 1;}
		//Լ - 
		if(Except_IsError(EXCEPT_DATA_BATTERY)){ob_self_check.result[SELF_CHECK_ITEM_BATTERY_INDEX] = SELF_CHECK_ITEM_FAIL;ob_self_check.item_fail_num = SELF_CHECK_ITEM_BATTERY_INDEX + 1;}
		//Լ - ѹ
		if(Except_IsError(EXCEPT_DATA_CHARGE)){ob_self_check.result[SELF_CHECK_ITEM_CHARGE_INDEX] = SELF_CHECK_ITEM_FAIL;ob_self_check.item_fail_num = SELF_CHECK_ITEM_CHARGE_INDEX + 1;}
		//Լ - 
		if(Except_IsError(EXCEPT_ANT_DAMAGE)){ob_self_check.result[SELF_CHECK_ITEM_ANT_INDEX] = SELF_CHECK_ITEM_FAIL;ob_self_check.item_fail_num = SELF_CHECK_ITEM_ANT_INDEX + 1;}					
		
		//Լʾʱͷתʱ
		if(ob_self_check.item_fail_num != 0){
			ob_self_check.led_switch = SELF_CHECK_LED_ON;
			ob_self_check.led_display_time = TIME_GetTicks();
			ob_self_check.led_toggle_time = TIME_GetTicks();
		}else{
			ob_self_check.led_switch = SELF_CHECK_LED_ON;
			if(self_uart_check_flag){
				LED_Start(LED_SELF_CHECK,COLOR_BLUE);
			}
			else{
				LED_Start(LED_SELF_CHECK,COLOR_CYAN);
			}
		}
}

static void ReadyIntoSelfMode(void){
		//ΪԼģʽתһ
		hal_ble_scan_result_Regist(&scan_reslt_handle);
		ob_self_check.is_scan_success 			= false;
		ob_self_check.scan_trigger_timeout 	= TIME_GetTicks();
	  ob_self_check.stage = SELF_CHECK_STAGE_READY;
		clear_all_except();	//쳣
		LED_Start(LED_SELF_CHECK,COLOR_BLACK); 
		LED_Stop(LED_SELF_CHECK);
		hal_ser_imu_mode_manage_set_required(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK, HAL_SER_IMU_MODE_MANAGE_IMU_MODE_ON);
		
		memset(ob_self_check.result, SELF_CHECK_ITEM_SUCCESS, SELF_CHECK_ITEMS);	
		SEGGER_RTT_printf(0,"app_self_checking_Process clear_all_except\r\n");
}


static void app_self_checking_Process(void)
{
	static uint8_t state =0;
	static uint8_t state_selft =0;
	static uint8_t displaycnt =0;

	switch(state){
		case 0:
			    if(BLE_Client_T_CHARGE_DONE == app_charge_Getstate() || \
						 BLE_Client_T_CHARGE_INSERT == app_charge_Getstate()
						){
							if(Except_IsErrorExist())																																														//ڴ
							{
								if(hal_ser_imu_mode_manage_get_ready(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK) != -1)																		//ǰԼ
								{
									hal_ser_imu_mode_manage_set_required(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK, HAL_SER_IMU_MODE_MANAGE_IMU_MODE_OFF);	//صԼ
								}
								else{																																																							//ǰԼ{
									ST_scan_start();
									ReadyIntoSelfMode();
									state =1;
									state_selft =0;
								}
							}
							else{
								//ΪԼģʽתһ
								ST_scan_start();
								ReadyIntoSelfMode();
								state =1;
								state_selft =0;
							}
					}
			    break;
    case 1:
					if(ob_self_check.is_scan_success == true)
					{
						ob_self_check.stage = SELF_CHECK_STAGE_CHECK;																				//Լ׶
						state =2;
						Process_SetHoldOn(app_self_checking_Process,1);
						SEGGER_RTT_printf(0,"ob_self_check.is_scan_success\r\n");
					}else{
             if((TIME_GetTicks() - ob_self_check.scan_trigger_timeout) > SELF_CHECK_SCAN_TRIGGER_TIMEOUT)		//ʱûɨ赽㲥
						 {
							  state = 2;
							  state_selft =3;
							  SEGGER_RTT_printf(0,"ob_self_check.scan_trigger_timeout\r\n");
						 }
					}
			    break;
		case 2:
			    if(BLE_Client_T_CHARGE_PULLOUT == app_charge_Getstate()){
						 ob_self_check.is_scan_success 			= false;																					//ɨɹ־λ	
			       hal_ble_scan_result_Clear(&scan_reslt_handle);																				//ֹɨ败
						 ST_scan_stop();
						 state = 0;
					}
			    break;
		default:
			    break;
	}
	
	switch(state_selft)
	{
		case 0:																															   
			  if(SELF_CHECK_STAGE_CHECK == ob_self_check.stage){//ɨɹ
					  MT_Run(500);
						if(-1 == hal_ser_imu_mode_manage_get_ready(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK)){
					       hal_ser_imu_mode_manage_set_required(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK, HAL_SER_IMU_MODE_MANAGE_IMU_MODE_ON);
						}
				    state_selft = 1;
					  ob_self_check.ImuSetTime_times = TIME_GetTicks();
				}
				break;
		case 1:
			  if(BLE_Client_T_CHARGE_PULLOUT == app_charge_Getstate()){
					 state_selft = 3;
				}
			  else {
						if(0 == hal_ser_imu_mode_manage_get_ready(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK)){
							state_selft = 2;
							displaycnt =0;
							app_self_check_getreslut();
						}
						else if(TIME_GetTicks() - ob_self_check.ImuSetTime_times >= SELF_CHECK_SET_IMU_TIMEOUT){
							Except_SetExceptype(EXCEPT_DATA_FRONT_ACC);
							Except_SetExceptype(EXCEPT_DATA_FRONT_GRY);
							Except_SetExceptype(EXCEPT_DATA_FRONT_MAG);
							Except_SetExceptype(EXCEPT_IMU_SUSPEND_OVERFLOW); 
							state_selft = 2;
							displaycnt =0;
						}
			  }
			  break;
	  case 2:
			   if(displaycnt < ob_self_check.item_fail_num){
						if(TIME_GetTicks() - ob_self_check.led_toggle_time >= SELF_CHECK_LED_TOGGLE_TIME){ob_self_check.led_toggle_time = TIME_GetTicks();
							if(ob_self_check.led_switch == SELF_CHECK_LED_ON){
								LED_Start(LED_SELF_CHECK,COLOR_LIGHRED);
								ob_self_check.led_switch = SELF_CHECK_LED_OFF;
							}
							else{
								LED_Start(LED_SELF_CHECK,COLOR_BLACK);
								ob_self_check.led_switch = SELF_CHECK_LED_ON;
								displaycnt++;
							}
						}
					}
				  else if(TIME_GetTicks() - ob_self_check.led_display_time >= SELF_CHECK_LED_DISPLAY_TIME){//ʾʱ䵽ˣֶȡԼʾ
						ob_self_check.led_display_time = TIME_GetTicks();
						displaycnt =0;
						app_self_check_getreslut();
					}
			    break;
		 case 3:
				 state_selft = 0;
		     ob_self_check.stage = SELF_CHECK_STAGE_READY;
         LED_Start(LED_SELF_CHECK,COLOR_BLACK); 
				 LED_Stop(LED_SELF_CHECK);
				 ob_self_check.led_switch = SELF_CHECK_LED_OFF;																					//ԼΪ
				 if(hal_ser_imu_mode_manage_get_ready(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK) != -1)        //ǰԼģʽ˳
				 {
						hal_ser_imu_mode_manage_set_required(HAL_SER_IMU_MODE_MANAGE_SELF_CHECK, HAL_SER_IMU_MODE_MANAGE_IMU_MODE_OFF);
				 }								
				 break;
		 default:
			   break;
	}	
}
static void app_self_checking_printf_process(void)
{
	SEGGER_RTT_printf(0,"self_check: FRONT_SENSOR:%d BACK_SENSOR:%d BATTERY:%d CHARGE:%d ANT:%d suspend_mode:%d scan_trigger_timeout:%d\n", \
	Except_IsError(EXCEPT_DATA_FRONT_ACC) || Except_IsError(EXCEPT_DATA_FRONT_GRY) || Except_IsError(EXCEPT_DATA_FRONT_MAG), \
	Except_IsError(EXCEPT_DATA_BACK_MAG), \
	Except_IsError(EXCEPT_DATA_BATTERY), \
	Except_IsError(EXCEPT_DATA_CHARGE), \
	Except_IsError(EXCEPT_ANT_DAMAGE), \
	Except_IsError(EXCEPT_IMU_SUSPEND_OVERFLOW),ob_self_check.scan_trigger_timeout);
}

/*API ----------------------------------------------------------*/
void app_self_checking_Init(void)
{
	ob_self_check.stage = SELF_CHECK_STAGE_READY;
	ob_self_check.ImuSetTime_times = 0;
	ob_self_check.item_fail_num = 0;
	ob_self_check.led_switch = SELF_CHECK_LED_OFF;
	ob_self_check.led_display_time = 0;
	ob_self_check.led_toggle_time = 0;
	ob_self_check.is_scan_success = false;																		
	ob_self_check.scan_trigger_timeout = SELF_CHECK_SCAN_TRIGGER_TIMEOUT;								

	
	Process_Start(SELF_CHECK_PROCESS_CYCLE,"app_self_checking_Process",app_self_checking_Process);
	Process_Start(300,"app_UartLine_Pair_Process",app_UartLine_Pair_Process);
	UART0_Rx_Regist(UART0_R_LINE_CHECK_ASK,cb_UART0_R_LINE_CHECK_ASK);
	UART0_Rx_Regist(UART0_R_LINE_CHECK_ACK,cb_UART0_R_LINE_CHECK_ACK);
//	Process_Start(1000,"app_self_checking_printf_process",app_self_checking_printf_process);
}

uint32_t app_self_checking_get_scan_trigger_timeout(void)
{
	return ob_self_check.scan_trigger_timeout;
}




















