
//flash 1.0V

/*********************************************************************
 * INCLUDES
 */
#include "bsp_flash.h"
#include "exception.h"
#include "system.h"

/*********************************************************************
 * LOCAL FUNCTIONS DEF
 */
static void fstorageCallbackFunc(nrf_fstorage_evt_t *pFstorageEvent);
static uint32_t getflashEndAddress(void);
static void waitForFlashReady(nrf_fstorage_t const *pFstorageHandle);

/*********************************************************************
 * LOCAL VARIABLES
 */
NRF_FSTORAGE_DEF(nrf_fstorage_t s_fstorageHandle) =
{
    /* Set a handler for fstorage events. */
    .evt_handler = fstorageCallbackFunc,

    /* These below are the boundaries of the flash space assigned to this instance of fstorage.
     * You must set these manually, even at runtime, before nrf_fstorage_init() is called.
     * The function nrf5_flash_end_addr_get() can be used to retrieve the last address on the
     * last page of flash available to write data. */
    .start_addr = START_FSTORAGE_ADDR,
    .end_addr   = END_FSTORAGE_ADDR,
};

/*********************************************************************
 * LOCAL FUNCTIONS
 */
/**
 @brief Fstorageдڴ
 @param flashAddr -[in] ַ
 @param readWriteFlag -[in] д־
 @param pData -[in&out] ָҪ
 @param dataLen -[in] ݳ
 @return 
*/
static uint32_t Fstorage_FlashRW(uint32_t flashAddr, uint8_t readWriteFlag, uint32_t *pData, uint32_t dataLen)
{
	ret_code_t errCode;
	
	if(flashAddr%4 != 0 || flashAddr == 0)return FLASH_ERROR_ADDRALIGN;
	if((flashAddr + dataLen) > END_FSTORAGE_ADDR){
		return FLASH_ERROR_ADDROVERBOUNDS;	//ǷԽ
	}
	
	if(readWriteFlag == FSTORAGE_READ)                       							// ȡ
	{
		errCode = nrf_fstorage_read(&s_fstorageHandle, flashAddr, pData, dataLen);
		if(errCode != NRF_SUCCESS)return FLASH_ERROR_READ;
	}
	else 																				// д
	{
		if(dataLen %4 != 0 || dataLen == 0)return FLASH_ERROR_DATAALIGN;
		if( FLASH_ZONEADDR_CHECK(flashAddr) )return FLASH_ERROR_ZONEADDR;
		
		errCode = nrf_fstorage_erase(&s_fstorageHandle, flashAddr, 1, NULL);			// ֻдλֵ1дλֵ0Ȳд	
		if(errCode != NRF_SUCCESS)return FLASH_ERROR_ERASE;
		errCode = nrf_fstorage_write(&s_fstorageHandle, flashAddr, pData, dataLen, NULL);
		if(errCode != NRF_SUCCESS)return FLASH_ERROR_WRITE;
	
		waitForFlashReady(&s_fstorageHandle);											// ȴд
	}
	
	return FLASH_OP_SUCCESS;
}

/**
@brief	FstorageдڴǰFLASHԤѲ
@param flashAddr -[in] ַ
@param pData -[in&out] ָҪ
@param dataLen -[in] ݳ
@param is_check - [in] Ƿݴ治
@return 0:дɹ  1:4ֽڲ  2:ݴ  3:Խ
*/
static uint32_t Fstorage_FlashOnlyWrite(uint32_t flashAddr, uint32_t *pData, uint32_t dataLen, bool is_check)
{
	ret_code_t errCode;
	uint32_t check,i,len;
	
	if(flashAddr%4 != 0 || flashAddr == 0)return FLASH_ERROR_ADDRALIGN;
	if(dataLen %4 != 0 || dataLen == 0)return FLASH_ERROR_DATAALIGN;
	if((flashAddr + dataLen) > END_FSTORAGE_ADDR){
		return FLASH_ERROR_ADDROVERBOUNDS;
	}
	
	if(is_check)
	{
		len = dataLen / 4;
		for(i=0;i<len;i++)
		{
			memcpy(&check, (uint32_t*)flashAddr + i, 4);
			if(check != 0xFFFFFFFF)return FLASH_ERROR_DATAEXIST;
		}
	}

	errCode = nrf_fstorage_write(&s_fstorageHandle, flashAddr, pData, dataLen, NULL);
	if(errCode != NRF_SUCCESS)return FLASH_ERROR_WRITE;
	
	waitForFlashReady(&s_fstorageHandle);											// ȴд
	
	return FLASH_OP_SUCCESS;
}

/**
 @brief Fstorageһҳ
 @param Ҫҳ׵ַ
 @return 
*/
static uint32_t Fstorage_FlashOnlyErasePage(uint32_t zone_addr)
{
	
	ret_code_t errCode;
	
	if(zone_addr%4 != 0 || zone_addr == 0)return FLASH_ERROR_ADDRALIGN;
	if( FLASH_ZONEADDR_CHECK(zone_addr) )return FLASH_ERROR_ZONEADDR;
	
	errCode = nrf_fstorage_erase(&s_fstorageHandle, zone_addr, 1, NULL);			// ֻдλֵ1дλֵ0Ȳд		
	if(errCode != NRF_SUCCESS)return FLASH_ERROR_ERASE;
	
	waitForFlashReady(&s_fstorageHandle);
	
	return FLASH_OP_SUCCESS;
}


/*********************************************************************
 * LOCAL FUNCTIONS
 */
/**
 @brief Fstorage¼ص
 @param pFstorageEvent -[in] Fstorage¼
 @return 
*/
static void fstorageCallbackFunc(nrf_fstorage_evt_t *pFstorageEvent)
{
    if(pFstorageEvent->result != NRF_SUCCESS)
    {
        SEGGER_RTT_printf(0,"--> Event received: ERROR while executing an fstorage operation.\n");
        return ;
    }

    switch(pFstorageEvent->id)
    {
	case NRF_FSTORAGE_EVT_WRITE_RESULT:
		//SEGGER_RTT_printf(0,"--> Event received: wrote %d bytes at address 0x%x.\n", pFstorageEvent->len, pFstorageEvent->addr);
		break;

	case NRF_FSTORAGE_EVT_ERASE_RESULT:
		//SEGGER_RTT_printf(0,"--> Event received: erased %d page from address 0x%x.\n", pFstorageEvent->len, pFstorageEvent->addr);
		break;

	default:
		break;
	}
}

/**
 @brief FlashϿдݵĵַ
 @param 
 @return 
*/
static uint32_t getflashEndAddress(void)
{
	uint32_t const bootloaderAddr = NRF_UICR->NRFFW[0];
	uint32_t const pageSz         = NRF_FICR->CODEPAGESIZE;
	uint32_t const codeSz         = NRF_FICR->CODESIZE;

  return (bootloaderAddr != 0xFFFFFFFF ? bootloaderAddr : (codeSz * pageSz));
}

/**
 @brief ȴд
 @param pFstorageHandle -[in] Fstorage
 @return 
*/
static void waitForFlashReady(nrf_fstorage_t const *pFstorageHandle)
{
    while(nrf_fstorage_is_busy(pFstorageHandle))										// While fstorage is busy, sleep and wait for an event.
    {
       nrf_pwr_mgmt_run();
    }
}

/**
 @brief flashȡN4ֽ
 @param addr-[in] ȡ׵ַ
 @param pData -[in&out] ָҪ
 @param dataLen -[in] ݳ
 @return 
*/
static uint32_t Read_N_4Byte_flash(uint32_t addr , uint32_t *pData, uint32_t dataLen)
{
	return Fstorage_FlashRW(addr, FSTORAGE_READ, pData, dataLen);
}

/**
 @brief ȡҳڵ׵ַ
 @param PageNum-[in] Ҫҵҳ
 @param PageAddr-[in/out] صҳ׵ַ
 @return 
*/
static uint32_t GetPageAddr(uint16_t PageNum, uint32_t *PageAddr)
{
	if(PageNum >= FLASH_PAGE_NUM)return FLASH_ERROR_PAGENUM;
	
	*PageAddr = FLASH_ZONE_ADDR_1 + PageNum * FLASH_PAGE_SIZE;
	
	return FLASH_OP_SUCCESS;
}

/**
 @brief ȡҳڵβַҳһҳ׵ַ
 @param PageNum-[in] Ҫҵҳ
 @param PageAddr-[in/out] صҳ׵ַ
 @return 
*/
static uint32_t GetPageEndAddr(uint16_t PageNum, uint32_t *PageAddr)
{
	if(PageNum >= FLASH_PAGE_NUM)return FLASH_ERROR_PAGENUM;
	
	*PageAddr = FLASH_ZONE_ADDR_1 + (PageNum+1) * FLASH_PAGE_SIZE;
	
	return FLASH_OP_SUCCESS;
}


/**
 @brief ȡַҳƫ
 @param addr-[in] Ҫҵflashַ
 @param _offset-[in/out] صƫ
 @return 
*/
static uint32_t GetDistanceFromPage(uint32_t addr, uint32_t *_offset)
{
	
	if(addr %4 != 0 || addr == 0)return FLASH_ERROR_ADDRALIGN;
	if(addr > END_FSTORAGE_ADDR || addr < START_FSTORAGE_ADDR){
		return FLASH_ERROR_ADDROVERBOUNDS;	//ǷԽ
	}
	
		//ȡƫ
	*_offset = ((addr - START_FSTORAGE_ADDR)%FLASH_PAGE_SIZE)/4;
	
	return FLASH_OP_SUCCESS;
}

/**
 @brief жǷҪ
 @param start_addr-[in] flash׵ַ
 @param len -[in] 
 @param is_erase-[in/out] 1Ҫ0Ҫ
 @return 
*/
static uint32_t __NeedErase(uint32_t start_addr , uint32_t len , uint8_t *is_erase)
{
	uint32_t err_code;
	uint32_t i = 0, data;
	
	if((start_addr + len) > END_FSTORAGE_ADDR)return FLASH_ERROR_ADDROVERBOUNDS;	
	if(start_addr%4 != 0 || start_addr == 0)return FLASH_ERROR_ADDRALIGN;
	if(len %4 != 0 || len == 0)return FLASH_ERROR_DATAALIGN;

	len /=4;
	for(;i<len;i++)
	{
		err_code = Fstorage_FlashRW(start_addr, FSTORAGE_READ, &data, 4);
		if(err_code != FLASH_OP_SUCCESS)return err_code;
		
		if(data != 0xFFFFFFFF)break;
		start_addr += 4;
	}
	
	*is_erase = (i==len)?0:1;
	
	return FLASH_OP_SUCCESS;
}

static void bsp_flash_init_process(void)
{
	if(Except_TxError(EXCEPT_FLASH_INIT,"bsp_flash_init_error\r\n") == 0)
	{
		Process_Stop(bsp_flash_init_process);
	}
}

/****************************************************ӿ****************************************************/

/**
 @brief Fstorageдڴʼ
 @param 
 @return 
*/
void Fstorage_FlashInit(void)
{
	int 				ret = 0;
	ret_code_t 	errCode;

	nrf_fstorage_api_t *pFstorageApi;
	pFstorageApi = &nrf_fstorage_sd;
	
	errCode = nrf_fstorage_init(&s_fstorageHandle, pFstorageApi, NULL);					// FlashĿʼַͽַʼ
	if(errCode != NRF_SUCCESS)ret = -1;
	
	(void)getflashEndAddress();															// ȡַжΪдַС
	
	if(ret == -1)
	{
		Process_Start(0,"bsp_flash_init_process",bsp_flash_init_process);
	}
}

/**
 @brief flashȡNֽ
 @param addr-[in] ȡ׵ַ
 @param pData -[in&out] ָҪ
 @param dataLen -[in] ݳ
 @return 
*/
uint32_t Read_N_Byte_flash(uint32_t addr , uint32_t *pData, uint32_t dataLen)
{
	if(addr > END_FSTORAGE_ADDR || addr < START_FSTORAGE_ADDR){
		return FLASH_ERROR_ADDROVERBOUNDS;	//ǷԽ
	}
	
	memcpy(pData, (uint32_t*)addr, dataLen);
	
	return FLASH_OP_SUCCESS;
}

/**
 @brief ȡflashַڵҳ
 @param addr-[in] Ҫҵflashַ
 @param page_num-[in/out] صҳ
 @return 
*/
uint32_t GetPageNum(uint32_t addr, uint16_t *PageNum)
{
	uint16_t page_num = 0;
	
	if(addr > END_FSTORAGE_ADDR || addr < START_FSTORAGE_ADDR){
		return FLASH_ERROR_ADDROVERBOUNDS;	//ǷԽ
	}
	
//	if(addr %4 != 0 || addr == 0)return FLASH_ERROR_ADDRALIGN;
	
	page_num = (addr - START_FSTORAGE_ADDR)/FLASH_PAGE_SIZE;
	if(page_num >= FLASH_PAGE_NUM)return FLASH_ERROR_PAGENUM;
	
	*PageNum = page_num;
	
	return FLASH_OP_SUCCESS;
}

/**
 @brief 1ҳ
 @param PageNum-[in] ҳ
 @return 
*/
uint32_t Erase_OnePage(uint16_t PageNum)
{
	uint32_t PageAddr;
	uint32_t err_code;
	
	err_code = GetPageAddr(PageNum, &PageAddr);
	if(err_code != FLASH_OP_SUCCESS)return err_code;
	
	return Fstorage_FlashOnlyErasePage(PageAddr);
}

/**
 @brief дNֵflash
 @param addr-[in] дflash׵ַ
 @param pdata-[in] Ҫд
 @param len-[in] ݳ 
 @return 
*/
uint32_t Write_N_4Byte_flash(uint32_t addr , uint32_t *pdata, uint32_t len)
{
	uint8_t  is_erase;				//жǷҪ
	uint16_t start_page = 0;	//ʼҳ
	uint32_t page_size = 0;		//ҪҳĴС
	uint32_t _offset = 0;			//ҳҳʼַ
	uint32_t page_offset = 0; //ҳƫ
	uint32_t data_offset = 0; //ƫ
	uint32_t remain_size = 0;	//βҳʹõʣС
	uint32_t base_addr = 0;		//βҳĻַ
	uint32_t surplus = 0;			//ܵʣֽ
	uint32_t err_code;				//
	uint32_t flash_PageBuff[PAGE_INT_SIZE];	//ҳ
	
	//ǷԽ
	if((addr + len) > END_FSTORAGE_ADDR){
		return FLASH_ERROR_ADDROVERBOUNDS;	
	}
	//ȡʼҳ
	err_code = GetPageNum(addr, &start_page);
	if(err_code != FLASH_OP_SUCCESS)return err_code;
	//ȡƫ
	err_code = GetDistanceFromPage(addr,&_offset);
	if(err_code != FLASH_OP_SUCCESS)return err_code;
	//Ҫдҳ
	if(((PAGE_INT_SIZE - _offset)*4) >= len)
	{
		page_size = 1;
		surplus = 0;
	}
	else
	{
		page_size = 1;
		surplus = len - ((PAGE_INT_SIZE - _offset)*4);
		page_size += ((surplus/FLASH_PAGE_SIZE) + (surplus%FLASH_PAGE_SIZE>0?1:0));
	}
	//ÿҳд
	surplus = len;
	for(page_offset = 0; page_offset < page_size; page_offset++)
	{
		
		//ȡҳ׵ַ
		err_code = GetPageAddr(start_page + page_offset, &base_addr);
		if(err_code != FLASH_OP_SUCCESS)return err_code;
		
		if(page_offset == 0)
		{
			//һƫƸĿĵַȷ
			remain_size = (PAGE_INT_SIZE - _offset)*4<surplus ? (PAGE_INT_SIZE - _offset)*4:surplus;
			surplus = len - remain_size;
		}
		else
		{
			//ƫƶǴ׵ַʼ
			_offset = 0;
			remain_size = surplus > FLASH_PAGE_SIZE ? FLASH_PAGE_SIZE : surplus;
			surplus = surplus - remain_size;
		}
		
		//жǷҪ
		err_code = __NeedErase(base_addr + _offset*4 , remain_size, &is_erase);
		if(err_code != FLASH_OP_SUCCESS)return err_code;
		
		
		//Ȳд
		if(is_erase)
		{
			//ȡ
			err_code = Read_N_4Byte_flash(base_addr , flash_PageBuff, FLASH_PAGE_SIZE);
			if(err_code != FLASH_OP_SUCCESS)return err_code;
			//޸
			memcpy(&flash_PageBuff[_offset], pdata + data_offset, remain_size);
			//д
			err_code = Fstorage_FlashRW(base_addr, FSTORAGE_WRITE, flash_PageBuff, FLASH_PAGE_SIZE);
			if(err_code != FLASH_OP_SUCCESS)return err_code;
		}
		else
		{
			//Ҫֱд
			err_code = Fstorage_FlashOnlyWrite(base_addr + _offset*4, pdata + data_offset, remain_size, false);	//ֱд
			if(err_code != FLASH_OP_SUCCESS)return err_code;
		}

		data_offset += remain_size/4;			//4ΪpDataָ+1ƶ4ֽ
		
	}
	
	return FLASH_OP_SUCCESS;
	
}

/**
 @brief ֻдNֵflashУǰFLASHԤѲ
 @param addr-[in] дflash׵ַ
 @param pdata-[in] Ҫд
 @param len-[in] ݳ 
 @return 
*/
uint32_t Only_Write_N_4Byte_flash(uint32_t addr , uint32_t *pdata, uint32_t len, bool is_check)
{
	return Fstorage_FlashOnlyWrite(addr, pdata, len, is_check);
}

/**
 @brief һҳпдĿռ䣨0xFF
 @param PageNum-[in] ҳ
 @param start_addr-[in/out] صĿпռʼַ
 @param len-[in/out] صĿռĳ
 @return 
*/
uint32_t Find_FreeSpace_In_Page(uint16_t PageNum, uint32_t *start_addr, uint32_t *len)
{
	uint32_t PageEndAddr,  RetrieveAddr, i;
	uint32_t flash_data;
	
	if(PageNum >= FLASH_PAGE_NUM)return FLASH_ERROR_PAGENUM;
	
	//ȡҳĽβַ
	GetPageEndAddr(PageNum, &PageEndAddr);
	
	//ҳβʼпռ
	RetrieveAddr = PageEndAddr;
	for(i = 0;i<PAGE_INT_SIZE;i++)
	{
		RetrieveAddr -= 4;
		Read_N_4Byte_flash(RetrieveAddr , &flash_data, 4);
		
		if(flash_data != 0xFFFFFFFF)break;
		
	}
	*start_addr = PageEndAddr - i *4;
	*len        = i*4;
	
	return FLASH_OP_SUCCESS;
}



/****************************************************END OF FILE****************************************************/
