#include "hal_flash.h"

/*********************************************************************
*DEFINITION
*/
typedef struct zone_s
{
	uint8_t	 page_num;
	uint32_t zone_start_addr;
}zone_t;


/*********************************************************************
* LOCAL VARIABLES
*/
static int16_t 		PageCount = FLASH_PAGE_NUM;											//ҳ
static uint32_t		zone_used_index = 0;														//zoneṹ±
static zone_t 		zone[FLASH_PAGE_NUM];														//zoneṹ

/*********************************************************************
 * LOCAL FUNCTIONS
 */
/**
 @brief ַͨȡ
 @param zone_addr[in] - ַ
 @param zone_index[out] - ط
 @return 
*/ 
static uint32_t Zone_GetIndex(uint32_t zone_addr, uint32_t *zone_index)
{
	for(int i=0; i < zone_used_index; i++)
	{
		if( zone[i].zone_start_addr <= zone_addr && (zone[i].zone_start_addr + zone[i].page_num * FLASH_PAGE_SIZE) > zone_addr ){
			if(zone_index != NULL)*zone_index = i;
			return ZONE_OP_SUCCESS;
		}
	}
	return ZONE_ERROR_NO_INDEX;
}


/*********************************************************************
* PUBLIC FUNCTIONS
*/
/**
 @brief ʼһ
 @param page_size[in] - ʹ˼ҳ
 @param zone_start_addr[out] - ط׵ַ
 @return 
*/ 
uint32_t Zone_Alloca(uint8_t page_num, uint32_t *zone_start_addr)
{
	if(page_num <= 0) return ZONE_ERROR_PARAM_PAGE_NUM;
	
	if(PageCount - page_num < 0)return ZONE_ERROR_NO_PAGE;
	
	zone[zone_used_index].page_num = page_num;
	if(zone_used_index == 0)zone[zone_used_index].zone_start_addr = FLASH_ZONE_ADDR_1;
	else zone[zone_used_index].zone_start_addr = zone[zone_used_index-1].zone_start_addr + FLASH_PAGE_SIZE * zone[zone_used_index-1].page_num;
	
	if(zone_start_addr != NULL)*zone_start_addr = zone[zone_used_index].zone_start_addr;
	
	PageCount -= page_num;
	zone_used_index++;

	
	return ZONE_OP_SUCCESS;
}

/**
 @brief ȡһָλõ
 @param destination_addr[in] - ȡĵַ
 @param pData[in] - ָ洢buff
 @param dataLen[in] - ȡĳȣλֽ
 @return 
*/ 
uint32_t Zone_Read(uint32_t destination_addr, uint32_t *pData, uint32_t dataLen)
{
	uint32_t 	err_code;						//
	uint32_t  zone_index;					//ǷӦ
	uint32_t  next_zone_addr;			//¸׵ַ
	
	//֤Ƿ˶Ӧ
	err_code = Zone_GetIndex(destination_addr, &zone_index);
	if(err_code != ZONE_OP_SUCCESS)return err_code;
	
	if(pData != NULL && dataLen != 0)
	{
		next_zone_addr = zone[zone_index].zone_start_addr + zone[zone_index].page_num * FLASH_PAGE_SIZE;
		//ܶһ
		if((destination_addr + dataLen) > next_zone_addr)return ZONE_ERROR_READ_FAIL;
		//ȡ
		err_code = Read_N_Byte_flash(destination_addr, pData, dataLen);
		if(err_code != FLASH_OP_SUCCESS)return ZONE_ERROR_READ_FAIL;
	}
	
	return ZONE_OP_SUCCESS;
	
}

/**
 @brief һָλдݣֿ֧ҳд
 @param destination_addr[in] - дĵַ
 @param pData[in] - ָ洢buff
 @param dataLen[in] - дĳ
 @return 
*/ 
uint32_t Zone_Write(uint32_t destination_addr, uint32_t *pData, uint32_t dataLen)
{
	uint32_t 	err_code;						//
	uint32_t  zone_index;					//ǷӦ
	uint32_t  next_zone_addr;			//¸׵ַ

	//֤Ƿ˶Ӧ
	err_code = Zone_GetIndex(destination_addr, &zone_index);
	if(err_code != ZONE_OP_SUCCESS)return err_code;
	
	if(pData != NULL && dataLen != 0)
	{
		next_zone_addr = zone[zone_index].zone_start_addr + zone[zone_index].page_num * FLASH_PAGE_SIZE;
		//дһ
		if((destination_addr + dataLen) > next_zone_addr)return ZONE_ERROR_WRITE_FAIL;
		//д
		err_code = Write_N_4Byte_flash(destination_addr, pData, dataLen);
		if(err_code != FLASH_OP_SUCCESS)return ZONE_ERROR_WRITE_FAIL;
	}
	return ZONE_OP_SUCCESS;
}
/**
 @brief һ
 @param zone_start_addr[in] - ׵ַ
 @return 
*/ 
uint32_t Zone_Erase(uint32_t zone_start_addr)
{
	uint32_t 	err_code;						//
	uint32_t  zone_index;					//ǷӦ
	uint16_t 	PageNum;
	
	//֤Ƿ˶Ӧ
	err_code = Zone_GetIndex(zone_start_addr, &zone_index);
	if(err_code != ZONE_OP_SUCCESS)return err_code;
	//һҳһҳ
	for(uint8_t i = 0; i < zone[zone_index].page_num; i++)
	{
		//ȡַӦҳ
		err_code = GetPageNum(zone[zone_index].zone_start_addr + i * FLASH_PAGE_SIZE, &PageNum);
		if(err_code != FLASH_OP_SUCCESS)return ZONE_ERROR_ERASE_FAIL;
		//ҳŲһҳ
		err_code = Erase_OnePage(PageNum);
		if(err_code != FLASH_OP_SUCCESS)return ZONE_ERROR_ERASE_FAIL;
	}
	return ZONE_OP_SUCCESS;
}

/**
 @brief ȡһֽڴС
 @param zone_start_addr[in] - ׵ַ
 @param bytes[out] - ֽڴС
 @return 
*/ 
uint32_t Zone_GetByteSize(uint32_t zone_start_addr, uint32_t *bytes)
{
	uint32_t 	err_code;						//
	uint32_t  zone_index;					//ǷӦ
	
	//֤Ƿ˶Ӧ
	err_code = Zone_GetIndex(zone_start_addr, &zone_index);
	if(err_code != ZONE_OP_SUCCESS)return err_code;
	
	if(bytes != NULL)*bytes = zone[zone_index].page_num * FLASH_PAGE_SIZE;
	
	return ZONE_OP_SUCCESS;
}




